library Nicklist;

{ Important note about DLL memory management: ShareMem must be the
  first unit in your library's USES clause AND your project's (select
  Project-View Source) USES clause if your DLL exports any procedures or
  functions that pass strings as parameters or function results. This
  applies to all strings passed to and from your DLL--even those that
  are nested in records and classes. ShareMem is the interface unit to
  the BORLNDMM.DLL shared memory manager, which must be deployed along
  with your DLL. To avoid using BORLNDMM.DLL, pass string information
  using PChar or ShortString parameters. }

uses
  SysUtils,
  Classes,
  WinFinder,
  Windows,
  Messages,
  DLLTools;

var
    W: TWinFinder;
    H: Integer;
    R: Integer;
    Handle: HWND;
    D: Pointer;

const
    NoInteger = 'Error: That''s not a valid integer value.';

{$R *.res}


function DefaultColor(Handle: HWND;New: Integer = -1): Integer;
var P:PChar;
    Map: HWND;
    ms: PChar;
begin
    ms := PChar('nicklist_default_color_'+inttostr(Handle));
    Map := OpenFileMapping(FILE_MAP_ALL_ACCESS,false,ms);
    if (Map = 0) then
        Map := CreateFileMapping(INVALID_HANDLE_VALUE,
                                 0,
                                 PAGE_READWRITE,
                                 0,
                                 100,
                                 ms);
    P := MapViewOfFile(Map, FILE_MAP_WRITE,0,0,0);
    if (New >= 0) then begin
        StrCopy(P,PChar(inttostr(New)));
        Result := DefaultColor(Handle);
    end
    else if (P = '') then Result := -1
    else begin
        try
            Result := strtoint(P);
        except
            on E:Exception do Result := -1;
        end;
    end;
end;

function Return(Text:String; Exit:Boolean = True):Boolean;
begin
    StrCopy(PChar(D^),PChar(Text));
    Result := Exit;
end;

function ListBox(mWnd, aWnd: HWND; data, parms: Pchar; show, nopause:boolean):integer; stdcall;
var S:PChar;
    M,X:String;
    I,N,PC:Integer;
begin
    Result := 3;
    D := @Data;
    PC := GetParameterCount(data,#32);
    if (PC < 2) then begin
        if (Return('Error: You need to specify at least 1 parameter.')) then exit;
    end;
    try
        H := strtoint(GetParameter(data,#32,1,1));
    except
        on E:Exception do begin
            if (Return('Error: That''s not a valid integer value.')) then exit;
        end;
    end;
    W := TWinFinder.Create(nil);
    W.InitWindowList(H);
    W.ExcludeAllBut(
        W.CreateFilter(WF_INVALID_HANDLE,WF_INVALID_HANDLE,'ListBox',WF_INVALID_STRING)
    );
    if (W.WinList.Count < 1) then begin
        if (Return('Error: Something is wrong with that hwnd.')) then exit;
    end;
    Handle := TWinObject(W.WinList[0]).hwnd;
    N := SendMessage(Handle,LB_GETCOUNT,0,0);
    M := UpperCase(GetParameter(data,#32,2,1));
    if (M = 'ALINE') then begin
        S := PChar(GetParameter(data,#32,3,PC-2));
        if (PC = 2) then S := '';
        R := SendMessage(Handle,LB_ADDSTRING,0,lparam(LongInt(S)));
        if (R = LB_ERR) then begin
            if (Return('Error. Something failed: '+ ErrorToString(GetLastError))) then exit;
        end;
        if (DefaultColor(H) >= 0) then
            SendMessage(Handle,LB_SETITEMDATA,R,DefaultColor(H));
        if Return('Done. New item index: '+inttostr(R)) then exit;
    end
    else if (M = 'ILINE') then begin
        S := PChar(GetParameter(data,#32,4,PC-3));
        try
            I := strtoint(GetParameter(data,#32,3,1));
        except
            on E:Exception do begin
                if (Return(NoInteger)) then exit;
            end;
        end;
        I := Limit(-1,I,N);
        if (PC = 3) then S := '';
        R := SendMessage(Handle,LB_INSERTSTRING,wParam(LongInt(I)),lparam(LongInt(S)));
        if (R = LB_ERR) then begin
            if (Return('Error. Something failed: '+ErrorToString(GetLastError))) then exit;
        end
        else begin
            if (DefaultColor(H) >= 0) then
                SendMessage(Handle,LB_SETITEMDATA,R,DefaultColor(H));
            if Return('Done. New item index: '+inttostr(R)) then exit;
        end;
    end
    else if (M = 'DLINE') then begin
        try
            I := strtoint(GetParameter(data,#32,3,1));
        except
            on E:Exception do begin
                if (Return(NoInteger)) then exit;
            end;
        end;
        I := Limit(0-N,I,N);
        if (I < 0) then I := N + I + 1;
        R := SendMessage(Handle,LB_DELETESTRING,lparam(LongInt(I)),0);
        if (R = LB_ERR) then begin
            if (Return('Error. Something failed: '+ErrorToString(GetLastError))) then exit;
        end
        else begin
            if Return('Done.') then exit;
        end;
    end
    else if (M = 'RSLINE') then begin
        try
            I := strtoint(GetParameter(data,#32,3,1));
            H := strtoint(GetParameter(data,#32,4,1));
        except
            on E:Exception do begin
                if (Return(NoInteger)) then exit;
            end;
        end;
        R := SendMessage(Handle,LB_SELITEMRANGE,wparam(true),makelparam(I,H));
        if (R = LB_ERR) then begin
            if (Return('Error. Something failed: '+ErrorToString(GetLastError))) then exit;
        end
        else begin
            if Return('Done.') then exit;
        end;
    end
    else if (M = 'CLINE') then begin
        try
            I := strtoint(GetParameter(data,#32,3,1));
            H := strtoint(GetParameter(data,#32,4,1));
        except
            on E:Exception do begin
                if (Return(NoInteger)) then exit;
            end;
        end;
        if (I < 0) then I := N - I;
        if (I > N) then begin
            if (Return('Error: That index is too high.')) then exit;
        end;
        R := sendmessage(Handle,LB_SETITEMDATA,I,H);
        if (R = LB_ERR) then begin
            if (Return('Error. Something failed: '+ErrorToString(GetLastError))) then exit;
        end
        else begin
            Return('Done.');
            R := SendMessage(handle,LB_GETSEL,I,0);
            SendMessage(handle,LB_SETSEL,wparam(True),I);
            if (R = 0) then SendMessage(handle,LB_SETSEL,wparam(false),I);
        end;
    end
    else if (M = 'DCOLOR') then begin
        try
            I := strtoint(GetParameter(data,#32,3,1));
        except
            on E:Exception do begin
                if (Return(NoInteger)) then exit;
            end;
        end;
        I := DefaultColor(H,I);
        if (I = -1) then begin
            if (Return('Error.')) then exit;
        end else begin
            if Return('Done. New default color: '+inttostr(I)) then exit;
        end;
    end
    else if (M = 'SCROLL') then begin
        if (PC < 3) then begin
            if (Return('Error: You need to specify at least 3 parameters.')) then exit;
        end;
        X := UpperCase(GetParameter(data,#32,3,1));
        case ord(copy(X,1,1)[1]) of
            43: R := 1;
            45: R := 2;
            61: R := 3;
            else begin
                if (Return('Error: First char must either be +, - or =.')) then exit;
            end;
        end;
        case ord(copy(X,2,1)[1]) of
            76: begin
                if (R = 1) then
                    R := SendMessage(Handle,WM_VSCROLL,MakeLParam(SB_LINEDOWN,0),0)
                else
                    R := SendMessage(Handle,WM_VSCROLL,MakeLParam(SB_LINEUP,0),0);
            end;
            80: begin
                if (R = 1) then
                    R := SendMessage(Handle,WM_VSCROLL,MakeLParam(SB_PAGEDOWN,0),0)
                else
                    R := SendMessage(Handle,WM_VSCROLL,MakeLParam(SB_PAGEUP,0),0);
            end;
            73: begin
                if (PC < 4) then begin
                    if (Return('Error: You need to specify at least 4 parameters.')) then exit;
                end;
                try
                    I := strtoint(GetParameter(data,#32,4,1));
                except
                    on E:Exception do begin
                        if (Return(NoInteger)) then exit;
                    end;
                end;
                R := SendMessage(handle,WM_VSCROLL,MakeLParam(SB_THUMBPOSITION,I),0);
            end;
            else begin
                if (Return('Error: Second char must either be L, P or I.')) then exit;
            end;
        end;
        if (R <> 0) then
            if (Return('Error. Something failed: '+ErrorToString(GetLastError))) then exit;
        if (Return('Done.')) then exit;
    end
    else if (M = 'TOPINDEX') then begin
        if (PC >= 3) then begin
            try
                I := strtoint(GetParameter(data,#32,3,1));
            except
                on E:Exception do begin
                    if (Return(NoInteger)) then exit;
                end;
            end;
            I := Limit(0,I,N-1);
            R := SendMessage(handle,LB_SETTOPINDEX,wparam(longint(I)),0);
            if (R = LB_ERR) then begin
                if (Return('Error. Something failed: '+ErrorToString(GetLastError)+inttostr(I))) then exit;
            end;
            R := SendMessage(handle,LB_GETTOPINDEX,wparam(I),0);
            if (Return('Done. New top index is '+inttostr(R))) then exit;
        end;
        R := SendMessage(handle,LB_GETTOPINDEX,wparam(I),0);
        if (Return('Done. Top index is '+inttostr(R))) then exit;
    end;
end;


exports
    ListBox;

begin
end.

