unit WinFinder;

interface

uses Windows, Classes, Contnrs, SysUtils;

const
    WF_INVALID_STRING = 'Invalid String';
    WF_INVALID_HANDLE = -1;

type TListSortOrder = 0..1;

type
    TWinObject = class(TObject)
        hwnd: hWnd;
        classname: string;
        title: string;
        parent: hWnd;
        public
            constructor Create; overload;
            constructor Create(hWnd: hWnd); overload;
            destructor Destroy; override;
    end;

type
    TWinFilterObject = class(TWinObject)
        hwnd: Integer;
        parent: Integer;
    end;

type
    TWinList = class(TObjectList)
        public
            procedure SortBy(SearchType: String;SortOrder: TListSortOrder);
    end;

type
    TWinFinder = class(TComponent)
        WinList: TWinList;
    private
        procedure FindWindows(Parent: hWnd; Rec: Boolean = True);
    public
        constructor Create(AOwner: TComponent); override;
        destructor Destroy(); override;
        procedure InitWindowList(Parent: hWnd);
        function CreateFilter(hWnd, Parent: Integer; Classname, Title: String): TWinFilterObject;
        procedure ExcludeAllBut(Filter: TWinFilterObject);
        procedure Exclude(Filter: TWinFilterObject);
    end;


procedure Register;

function CaseFake(Substr:String;S:Array of String):Integer;
function GetWindowClass(hWnd: hwnd):String;
function GetWindowTitle(hWnd: hwnd):String;

function SortByHandleCallBack(Item1, Item2: Pointer): Integer;



implementation

{ Component Stuff }

procedure Register;
begin
  RegisterComponents('Win32', [TWinFinder]);
end;
{ Misc }

function CaseFake(Substr:String;S:Array of String):Integer;
var I:Integer;
begin
    For I:=0 to Length(S)-1 do begin
        if (S[I] = Substr) then begin
            Result := I;
            Exit;
        end;
    end;
    Result := -1;
end;

function GetWindowClass(hWnd: hwnd):String;
var P:PChar;
begin
    GetMem(P,256);
    GetClassName(hWnd,P,256);
    Result := P;
    FreeMem(P,256);
end;

function GetWindowTitle(hWnd: hwnd):String;
var P:PChar;
begin
    GetMem(P,GetWindowTextLength(hWnd)+1);
    GetWindowText(hWnd,P,256);
    Result := P;
    FreeMem(P,GetWindowTextLength(hWnd)+1);
end;

{ Sortshit }

function SortByHandleCallBack(Item1, Item2: Pointer): Integer;
begin Result := TWinObject(Item2).hwnd - TWinObject(Item1).hwnd; end;

function SortByHandleDescCallBack(Item1, Item2: Pointer): Integer;
begin Result := TWinObject(Item1).hwnd - TWinObject(Item2).hwnd; end;

function SortByParentCallBack(Item1, Item2: Pointer): Integer;
begin Result := TWinObject(Item2).parent - TWinObject(Item1).parent; end;

function SortByParentDescCallBack(Item1, Item2: Pointer): Integer;
begin Result := TWinObject(Item1).parent - TWinObject(Item2).parent; end;

function SortByClassCallBack(Item1, Item2: Pointer): Integer;
begin Result := CompareText(TWinObject(Item1).Classname,TWinObject(Item2).Classname); end;

function SortByClassDescCallBack(Item1, Item2: Pointer): Integer;
begin Result := CompareText(TWinObject(Item2).Classname,TWinObject(Item1).Classname); end;

function SortByTitleCallBack(Item1, Item2: Pointer): Integer;
begin Result := CompareText(TWinObject(Item1).Title,TWinObject(Item2).Title); end;

function SortByTitleDescCallBack(Item1, Item2: Pointer): Integer;
begin Result := CompareText(TWinObject(Item2).Title,TWinObject(Item1).Title); end;


{ TWinObject }

constructor TWinObject.Create;
begin
    inherited Create;
end;

constructor TWinObject.Create(hWnd: hWnd);
begin
    inherited Create;
    Self.hwnd := hWnd;
    Self.parent := GetParent(hWnd);
    Self.classname := GetWindowClass(hWnd);
    Self.title := GetWindowTitle(hWnd);
end;

destructor TWinObject.Destroy;
begin
    inherited Destroy;
end;




{ TWinList }


procedure TWinList.SortBy(SearchType: String;SortOrder: TListSortOrder);
begin
    case CaseFake(SearchType,['Handle','Parent','Title','Class']) of
    0:  begin
            case SortOrder of
                0: Sort(SortByHandleCallBack);
                1: Sort(SortByHandleDescCallBack);
            end;
        end;
    1:  begin
            case SortOrder of
                0: Sort(SortByParentCallBack);
                1: Sort(SortByParentDescCallBack);
            end;
        end;
    2:  begin
            case SortOrder of
                0: Sort(SortByTitleCallBack);
                1: Sort(SortByTitleDescCallBack);
            end;
        end;
    3:  begin
            case SortOrder of
                0: Sort(SortByClassCallBack);
                1: Sort(SortByClassDescCallBack);
            end;
        end;
    end;

end;

{ TWinFinder }

constructor TWInFinder.Create(AOwner: TComponent);
begin
    inherited Create(AOwner);
    WinList := TWinList.Create;
end;

destructor TWInFinder.Destroy();
begin
    inherited Destroy;
end;

procedure TWinFinder.FindWindows(Parent: hWnd; Rec: Boolean = True);
var I: hWnd;
begin
    I := 0;
    while (true) do begin
        I := FindWindowEx(Parent,I,nil,nil);
        if (i = 0) then Break;
        WinList.Add(TWinObject.Create(i));
        if (Rec) then FindWindows(I, Rec);
    end;
end;

procedure TWinFinder.InitWindowList(Parent: hWnd);
begin
    FindWindows(Parent);
end;

function TWinFinder.CreateFilter(hWnd, Parent: Integer; Classname, Title: String): TWinFilterObject;
begin
    Result := TWinFilterObject.Create;
    Result.hwnd := hWnd;
    Result.classname := Classname;
    Result.parent := parent;
    Result.title := title;
end;

procedure TWinFinder.ExcludeAllBut(Filter: TWinFilterObject);
var I:Integer;
    W: TWinObject;
begin
    I := 0;
    while i < WinList.Count-1 do begin
        W := TWinObject(WinList[I]);
        if ((Integer(W.hwnd) <> Filter.hwnd) and
            (Filter.hwnd <> WF_INVALID_HANDLE)) or
           ((Integer(W.parent) <> Filter.parent) and
            (Filter.parent <> WF_INVALID_HANDLE)) or
           ((UpperCase(W.classname) <> UpperCase(Filter.classname)) and
            (Filter.classname <> WF_INVALID_STRING)) or
           ((UpperCase(W.title) <> UpperCase(Filter.title)) and
            (Filter.title <> WF_INVALID_STRING)) then begin
                WinList.Delete(I);
                Dec(I);
           end;
        Inc(I);
    end;
end;

procedure TWinFinder.Exclude(Filter: TWinFilterObject);
var I:Integer;
    W: TWinObject;
begin
    I := 0;
    while i < WinList.Count-1 do begin
        W := TWinObject(WinList[I]);
        if ((Integer(W.hwnd) = Filter.hwnd) and
            (Filter.hwnd <> WF_INVALID_HANDLE)) or
           ((Integer(W.parent) = Filter.parent) and
            (Filter.parent <> WF_INVALID_HANDLE)) or
           ((UpperCase(W.classname) = UpperCase(Filter.classname)) and
            (Filter.classname <> WF_INVALID_STRING)) or
           ((UpperCase(W.title) = UpperCase(Filter.title)) and
            (Filter.title <> WF_INVALID_STRING)) then begin
                WinList.Delete(I);
                Dec(I);
           end;
    inc(I);
    end;
end;

end.
